
//Zuordnung der Pins:
//Anzeige   Mikrocontroller
//
//1         PA6
//2         PA5
//3         (PB2) Doppelpunkt GENDERT: jetzt KanalA bei Drehgeber!
//4         PB1
//5         PA2
//6         PC3
//7         PA1
//8         PC4
//9         PC5
//10        PB3
//11        PC7
//12        PC6
//
//Auswahl der Ziffern: (Auswahl durch +5V an entsprechenden digitalen Ausgngen)
//Ziffer   Anzeige Mikrocontroller
//10^0     6       PC3
//10^1     8       PC4
//10^2     9       PC5
//10^3    12       PC6
//
//Auswahl der Segmente
//
// A
//F B
// G
//E C
// D
//Segment  Anzeige Mikrocontroller
//A        11      PC7
//B         7      PA1
//C         4      PB1
//D         2      PA5
//E         1      PA6
//F        10      PB3
//G         5      PA2
//DP        3     (PB2) Doppelpunkt GENDERT: jetzt KanalA bei Drehgeber!

class Anzeige  :public CoachBASIS, AnzeigeBASIS
{
    private:
        uint8_t zaehler;
    public: 

        Anzeige()
		{
		}

        void start()
		{
            zaehler=0;        
            //Bentigte Pins als Ausgang setzen:
            DDRA |= 0b01100110;
            DDRB |= 0b00001010;
            DDRC |= 0b11111000;
		}
		
        void stop()
        {
            DDRA &= ~0b01100110;
            DDRB &= ~0b00001010;
            DDRC &= ~0b11111000;
        }

        void zahl (int wert)
        {
            //zu Beginn, da manche Bedingungen aus Funktion herausspringen:
            zaehler++;
            zaehler%=4;

            //Auswahl des Anzeigelements gem Zaehlerstand treffen:
            PORTC&=~( (1<<PC3) | (1<<PC4) | (1<<PC5) | (1<<PC6) ) ;
            PORTC|=(1<<(zaehler%4+3));

            //alle aus (+5V):
            PORTA|=( (1<<PA1) | (1<<PA2) | (1<<PA5) | (1<<PA6) );
            PORTB|=( (1<<PB1) | (1<<PB3) );
            PORTC|=(1<<PB7);

            //Nicht darstellbare Zahlen abfangen:
            //---- als Unterstrich fr <-999
            //---- als Oberstrich fr >9999
            if(wert<-999)
            {
                PORTA&=~(1<<PA5);
                return;
            }
            if(wert>9999)
            {
                PORTA&=~(1<<PC7);
                return;
            }

            //Wenn negativ und zaehler==3, Minuszeichen zeigen
            if(zaehler==3 && wert<0)
            {
                PORTA&=~(1<<PA2);
                return;
            }

            //Ziffer extrahieren:
            if(zaehler>0)
                 wert/=10;
            if(zaehler>1)
                 wert/=10;
            if(zaehler>2)
                 wert/=10;
            if(wert<0)
                 wert=-wert;
            wert%=10;

            //Ziffer eintragen:
            switch(wert)
            {
                case 0:
                    PORTA&=~( (1<<PA1) | (0<<PA2) | (1<<PA5) | (1<<PA6) );
                    PORTB&=~( (1<<PB1) | (1<<PB3) );
                    PORTC&=~(1<<PC7);
                break;
                case 1:
                    PORTA&=~( (1<<PA1) | (0<<PA2) | (0<<PA5) | (0<<PA6) );
                    PORTB&=~( (1<<PB1) | (0<<PB3) );
                    PORTC&=~(0<<PC7);
                break;
                case 2:
                    PORTA&=~( (1<<PA1) | (1<<PA2) | (1<<PA5) | (1<<PA6) );
                    PORTB&=~( (0<<PB1) | (0<<PB3) );
                    PORTC&=~(1<<PC7);
                break;
                case 3:
                    PORTA&=~( (1<<PA1) | (1<<PA2) | (1<<PA5) | (0<<PA6) );
                    PORTB&=~( (1<<PB1) | (0<<PB3) );
                    PORTC&=~(1<<PC7);
                break;
                case 4:
                    PORTA&=~( (1<<PA1) | (1<<PA2) | (0<<PA5) | (0<<PA6) );
                    PORTB&=~( (1<<PB1) | (1<<PB3) );
                    PORTC&=~(0<<PC7);
                break;
                case 5:
                    PORTA&=~( (0<<PA1) | (1<<PA2) | (1<<PA5) | (0<<PA6) );
                    PORTB&=~( (1<<PB1) | (1<<PB3) );
                    PORTC&=~(1<<PC7);
                break;
                case 6:
                    PORTA&=~( (0<<PA1) | (1<<PA2) | (1<<PA5) | (1<<PA6) );
                    PORTB&=~( (1<<PB1) | (1<<PB3) );
                    PORTC&=~(1<<PC7);
                break;
                case 7:
                    PORTA&=~( (1<<PA1) | (0<<PA2) | (0<<PA5) | (0<<PA6) );
                    PORTB&=~( (1<<PB1) | (0<<PB3) );
                    PORTC&=~(1<<PC7);
                break;
                case 8:
                    PORTA&=~( (1<<PA1) | (1<<PA2) | (1<<PA5) | (1<<PA6) );
                    PORTB&=~( (1<<PB1) | (1<<PB3) );
                    PORTC&=~(1<<PC7);
                break;
                case 9:
                    PORTA&=~( (1<<PA1) | (1<<PA2) | (1<<PA5) | (0<<PA6) );
                    PORTB&=~( (1<<PB1) | (1<<PB3) );
                    PORTC&=~(1<<PC7);
                break;
            }
        }	
		
	
	
};

