#define TAKTFREQUENZ 9216000
#define BAUDRATE 115200
#define PUFFERGROESSE 30

class RS232		: public RS232BASIS, CoachBASIS
{
	private:
		uint16_t baudregister;
        char  puffer[PUFFERGROESSE];
        int16_t  i,k;
        double   x,y;
	public:	    
        void start()
        {
            baudregister = TAKTFREQUENZ/8/BAUDRATE-1;
            UBRRH = (unsigned char) (baudregister>>8); //Setzen des HIGH-Bytes des Baudraten-Registers
            UBRRL = (unsigned char)  baudregister;     //Setzen des LOW -Bytes des Baudraten-Registers
            //Einschalten des Senders und des Empfngers
            UCSRB = (1<<TXEN) | (1<<RXEN);	        
            //Setzen des Nachrichtenformats: 8 Datenbits, 1 Stopbits
            UCSRC = (1<<URSEL)|(1<<UCSZ0)|(1<<UCSZ1);
            UCSRA = (1<<U2X);
        }
    
        void stop()
        {
            UCSRB &= ~( (1<<TXEN) | (1<<RXEN) );	        
        }

        void sendeZeichen(uint8_t zeichen)
        {
            while( !(UCSRA & (1<<UDRE)) ); 
            UDR = zeichen;
        }

        uint8_t holeZeichen()
        {
			while(!(UCSRA & (1<<RXC)));
			return UDR;
        }

        void sendeText (char* text)
        {
            i=0;
            while(text[i]!='\0')
            {
                sendeZeichen(text[i]);								
                i++;
            }
        }
	
        char* holeText(bool reflect)
        {
            i=0;
            do
            {
                puffer[i]=holeZeichen();
                if(reflect==true)
                    sendeZeichen(puffer[i]);
                i++;
            } while(i<PUFFERGROESSE-1 && puffer[i-1]!='#');
            puffer[i-1]='\0';
            return puffer;
        }		

        void sendePositiveInteger(int32_t zahl)
        {
            i=0;            
            do
            {
                puffer[i]=48+zahl%10;
                zahl/=10;
                i++;
            } while(zahl>0);

            for(k=0;k<i;k++)
                sendeZeichen(puffer[i-k-1]);
        }

        void sendeInteger(int16_t zahl)
        {

            if (zahl<0)
            {
                sendeZeichen('-');
                zahl=-zahl;
            }
            sendePositiveInteger(zahl);
       }

       int16_t holeInteger(bool reflect)
       {				
           holeText(reflect);
           i=0;
           k=0;
           if(puffer[0]=='-')
               i++;
           while(i<PUFFERGROESSE && puffer[i]>=48 && puffer[i]<=57)
           {
               k*=10;
               k+=puffer[i]-48;
               i++;
           }
           if(puffer[0]=='-')
               k=-k;
           return k;
       }	

       void sendeDouble(double zahl)
       {
            if (zahl<0)
            {
                sendeZeichen('-');
                zahl=-zahl;
            }
            sendePositiveInteger((long)zahl);
            sendeZeichen('.');
            sendePositiveInteger((long)((zahl-(double)((int)zahl))*10000.0));
       }	

       double holeDouble(bool reflect)
       {
           holeText(reflect);
           i=0;
           k=0;
           if(puffer[0]=='-')
               i++;
           while(i<PUFFERGROESSE && puffer[i]>=48 && puffer[i]<=57)
           {
               k*=10;
               k+=puffer[i]-48;
               i++;
           }
           if(puffer[0]=='-')
               k=-k;
           x=(double)k;
           if(puffer[i]=='.')
           {
               i++;
               k=0;
               while(i<PUFFERGROESSE && puffer[i]>=48 && puffer[i]<=57)
               {
                   k*=10;
                   k+=puffer[i]-48;
                   i++;
               }                              
           }
           y=(double)k;
           while(y>1.0)
               y*=0.1;
           if(puffer[0]=='-')
               return x-y;
           return x+y;
       }
};








