﻿//Projektwoche "COACH"
//Projektgruppe 3
//
//Skript optimiert das Antriebsmodell des COACH-Vehikels entsprechend der aufgezeichneten Sprungantwort
//
//es werden 2 Optimierungsverfahren angewendet:
//    a) die Scilab-Funktion optim()
//    b) ein selbst erstelltes optimiertes Gradienten-Verfahren
//
//ausgegeben wird für jedes Verfahren der optimale Parametersatz und die zugehörige Güte
//zusätzlich erfolgt eine grafische Darstellung Sprungantworten

clear;
clc;
funcprot(0);

//*** Einlesen der vorgegebenen Sprungantwort ***
//*   benötigt:  n - name und Pfad der zu lesenden Textdatei (enthält Spaltenweise Zeit und Sprungantwort)
//*              dt- dt>0 gewünschte Schrittweite, für Interpolation
//*                  dt=0 keine Interpolation
//*              m - maximale Zeit für Simulation
//*
//*   liefert:   t - Vektor der Zeit (mit der geünschten Schrittweite dt)
//*              y - Vektor der interpolierten Funktionswerte der Sprungantwort

function [t,y] = Einlesen(n,dt,m)
  soll=fscanfMat(n);
  if (dt>0)                              //interpolieren:
    ende=soll(length(soll(:,1)),1);      //Intervall auf m begrenzen
    if (ende>m) then
      ende=m;
    end
    t=soll(1,1):dt:ende;
  else                                  //nicht interpolieren,dann Zeitvektor im Original übernehmen
    if (soll(length(soll(:,1)),1)>m)    //aber nur solange die Werte kleiner sind als m
      z=0;
      while(0==0)
        z=z+1
        if (soll(z,1)>m) break; end;
      end
      t=soll(1:z-1,1)';
    else
      t=soll(:,1)';
    end
  end
  y=interp1(soll(:,1),soll(:,2),t,'linear');
endfunction

//*** Berechnet einen Zeitschritt des Antrieb-Modells ***
//*   wird von ode-Funktion aufgerufen
//*   Soll-Wert-Vorgabe durch Konstante
function f = rechteSeite(t,y)
  soll=1023;
  f(1,1)=   y(2,1);
  f(2,1)=aKoeff(1,1)*y(1,1)+aKoeff(2,1)*y(2,1)+aKoeff(3,1)*soll;
endfunction

//*** Liefert die Güte eines Parametersatzes als quadratische Abweichung ***
//*   Parameter müssen in globalem aKoeff hinterlegt sein
//*   Funktion benötigt KEINE konst. Schrittweite dt
function guete = berechne_guetewert()
  fun = ode(y0,t0,t_ziel,rechteSeite);
  y_ist = fun(1,:);
  guete=sum( ((y_ist-y_ziel)^2).*dt );
endfunction

//*** Liefert die Güte für die Scilab-optim-Funktion
function fehler = berechneFehler(parameter)
    aKoeff = parameter;
    y = ode(y0,t0,t_ziel,rechteSeite);
    [zeilen,spalten] = size(y);
    fehler = sum( (( y(1,1:spalten) - y_ziel )^2).*dt );
endfunction

//**************************************Hauptprogramm********************************************

//reale Sprungantwort einlesen
[t_ziel,y_ziel]=Einlesen('E:\Projektwoche\02_Ergebnisse_Sprungantworten\A4_900.txt',0.1,2.0);

//für jede Simulationsschritt dt berechnen (falls nicht konstant)
//wird in den Güte-Berechnungen verwendet
t2 = 0;
t2(2:length(t_ziel)+1) = t_ziel;
dt = t_ziel - t2(1:length(t2)-1);

//Modell-Variablen
y0=[0;0];              //Anfangsbedingungen
t0=0;                  //Anfangszeit

//Variablen für Optimierer
Opti_Param=[-2;-1;1];  //Startwerte für Parameter-Suche [-2;-1;1]
aKoeff=Opti_Param;     //Übergabepuffer für Modell

Anzahl_Parameter=length(Opti_Param);

Korrekturen=[0;          //delta-Schritte die pro Durchlauf getestet werden:
             20 ;-20 ;   //1-fache Schrittweite
             10 ;-10  ;  //halbe Schrittweite
             40 ;-40 ];  //doppelte Schrittweite
Test_Anzahl=length(Korrekturen);
for i=1:Anzahl_Parameter  //für jeden Parameter eine Spalte erstellen
  Korrektur_delta(:,i)=Korrekturen;
end

Anz_Optimierer_Schritte=30;  //Abbruchbedingung

//Versuch 1: interner Optimierer
[fehlermin,popt] = optim(list(NDcost,berechneFehler),Opti_Param);

//Versuch 2: optim. Gradienten Verfahren
for j = 1:Anz_Optimierer_Schritte

  for k = 1:Anzahl_Parameter 
    Opti_werte_test=zeros(Test_Anzahl);            //speichert die Ergebnisse der Versuche für den aktuellen Parameter
    aKoeff=Opti_Param;                             //Parametersatz auf Anfangswerte setzen

    for m = 1:Test_Anzahl                          //alle delta-Möglichkeiten testen
      aKoeff(k,1)=Opti_Param(k,1)+Korrektur_delta(m,k);
      Opti_werte_test(m,1)=berechne_guetewert();   //Guete jedes Mal speichern
    end  
  
    [best_wert,best_index]=min(Opti_werte_test);   //Besten index raussuchen
    Opti_Param(k,1)=Opti_Param(k,1)+Korrektur_delta(best_index,k);  //...Parameter für nächsten Durchlauf dauerhaft ändern

    if ((best_index==3) | (best_index==4))         //halber delta-Schritt hat bestes Ergebnis gebracht?
      Korrektur_delta(:,k)=Korrektur_delta(:,k)/2; //...dann Schrittweite generell halbieren für's nächste Mal
    end

    if ((best_index==5) | (best_index==6))         //doppelter delta-Schritt hat bestes Ergebnis gebracht?
      Korrektur_delta(:,k)=Korrektur_delta(:,k)/2; //...dann Schrittweite generell verdoppeln für's nächste Mal
    end
  end;
   
end;

//Darstellung der Ergebnisse
xtitle("Optimierung COACH Antriebsmodell","Zeit [s]","Omega_Viertel [1/4 rad/sek]");

//erst vom internen Optimierer
aKoeff=popt;
Koeffizienten_optim_Funk=popt
Guete_optim_Funktion=berechne_guetewert()
fun = ode(y0,t0,t_ziel,rechteSeite);
plot(t_ziel,fun(1,:),'r');

//dann von opt. Grad.Verf.
aKoeff=Opti_Param;
Koeffizienten_oGrad_Verf=Opti_Param
Guete_o_Grad_Verf=berechne_guetewert()
fun = ode(y0,t0,t_ziel,rechteSeite);
plot(t_ziel,fun(1,:),'b');

//zuletzt aufgezeichnete Sprungantwort plotten
plot(t_ziel,y_ziel,'m')

legend("Optimiertes Modell 1 (optim-Funktion)","Optimiertes Modell 2 (o.Grad.Verf.)","Sprungantwort COACH",4);