﻿//Projektwoche "COACH"
//Projektgruppe 3
//
//Skript optimiert den Regler für das Antriebsmodell des Vehikels
//
//es werden 2 Optimierungsverfahren angewendet:
//    a) die Scilab-Funktion optim()
//    b) ein selbst erstelltes optimiertes Gradienten-Verfahren
//
//als Zielfunktion dient eine konstante Zielgröße für eine einzustellende Geschwindigkeit

//ausgegeben wird für jedes Verfahren der optimale Parametersatz und die zugehörige Güte
//zusätzlich erfolgt eine grafische Darstellung

clear;
clc;
funcprot(1);

//*** Berechnet einen Zeitschritt des Antrieb-Modells ***
//*   wird von ode-Funktion aufgerufen
//*   Soll-Wert-Vorgabe zu jedem Zeitpunkt durch Vektor y_ziel
function f = rechteSeite(t,y)
  index=1+(t/deltaT);
  e=y_ziel(index)-y(1,1);         //Regeldifferenz
 // e=15-y(1,1);
  PAnteil=rKoeff(1,1)*e;
  DAnteil=rKoeff(2,1)*(-y(2,1));
  IAnteil=rKoeff(3,1)*( y(3,1));

  if (IAnteil> 10000) IAnteil= 10000; end;
  if (IAnteil<-10000) IAnteil=-10000; end;

  reG=PAnteil+DAnteil+IAnteil;

  if (reG> 1023) reG= 1023; end;
  if (reG<-1023) reG=-1023; end;
    
  f(1,1)=y(2,1);
  f(2,1)=aKoeff(1,1)*y(1,1)+aKoeff(2,1)*y(2,1)+aKoeff(3,1)*reG;
  f(3,1)=e;                  //Integral der Regeldifferenz (wird von Integrator gebildet)
endfunction

//*** Liefert die Güte eines Parametersatzes als quadratische Abweichung ***
//*   Parameter müssen in globalem rKoeff hinterlegt sein
//*   Funktion benötigt KEINE konst. Schrittweite dt
function guete = berechne_guetewert()
  fun = ode(y0,t0,t_ziel,rechteSeite);
  y_ist = fun(1,:);
  guete=sum( ((y_ist-y_ziel)^2).*dt );
endfunction

//*** Liefert die Güte für die Scilab-optim-Funktion
function fehler = berechneFehler(parameter)
    rKoeff = parameter;
    y = ode(y0,t0,t_ziel,rechteSeite);
    [zeilen,spalten] = size(y);
    fehler = sum( (( y(1,1:spalten) - y_ziel )^2).*dt );
endfunction

//**********************************Hauptprogramm************************************************

//Zielgröße y_ziel abhängig von Zeit erstellen
deltaT=0.1;
soll=15;
t_ziel=0:deltaT:5;
y_ziel=soll*ones(1,length(t_ziel));
//y_ziel(1,(length(t_ziel)/2):length(t_ziel))=soll/2;

//für jede Simulationsschritt dt berechnen (falls unterschiedlich)
//wird in den Güte-Berechnungen verwendet
t2 = 0;
t2(2:length(t_ziel)+1) = t_ziel;
dt = t_ziel - t2(1:length(t2)-1);

//Modell-Variablen
y0=[0;0;0];            //Anfangsbedingungen
t0=0;                  //Anfangszeit
aKoeff=[-42;-7;1];     //Parameter für Modell

//Variablen für Optimierer
Opti_Param=[300;300;300]; //Startwerte für Parameter-Suche
rKoeff=Opti_Param;     //Übergabepuffer für PID-Regel-Model

Anzahl_Parameter=length(Opti_Param);

Korrekturen=[0;         //delta-Schritte die pro Durchlauf getestet werden:
             20 ;-20 ;  //1-fache Schrittweite
             10 ;-10;   //halbe Schrittweite
             40 ;-40 ]; //doppelte Schrittweite
Test_Anzahl=length(Korrekturen);
for i=1:Anzahl_Parameter  //für jeden Parameter eine Spalte erstellen
  Korrektur_delta(:,i)=Korrekturen;
end

Anz_Optimierer_Schritte=10;  //Abbruchbedingung

//Versuch 1: interner Optimierer
[fehlermin,popt] = optim(list(NDcost,berechneFehler),Opti_Param);
//popt=Opti_Param;

//Versuch 2: optim. Gradienten Verfahren
for j = 1:Anz_Optimierer_Schritte

  for k = 1:Anzahl_Parameter 
    Opti_werte_test=zeros(Test_Anzahl);            //speichert die Ergebnisse der Versuche für den aktuellen Parameter
    rKoeff=Opti_Param;                             //Parametersatz auf Anfangswerte setzen

    for m = 1:Test_Anzahl                          //alle delta-Möglichkeiten testen
      rKoeff(k,1)=Opti_Param(k,1)+Korrektur_delta(m,k);
      Opti_werte_test(m,1)=berechne_guetewert();   //Guete jedes Mal speichern
    end  
  
    [best_wert,best_index]=min(Opti_werte_test);   //Besten index raussuchen
    Opti_Param(k,1)=Opti_Param(k,1)+Korrektur_delta(best_index,k);  //...Parameter für nächsten Durchlauf dauerhaft ändern

    if ((best_index==3) | (best_index==4))         //halber delta-Schritt hat bestes Ergebnis gebracht?
      Korrektur_delta(:,k)=Korrektur_delta(:,k)/2; //...dann Schrittweite generell halbieren für's nächste Mal
    end

    if ((best_index==5) | (best_index==6))         //doppelter delta-Schritt hat bestes Ergebnis gebracht?
      Korrektur_delta(:,k)=Korrektur_delta(:,k)/2; //...dann Schrittweite generell verdoppeln für's nächste Mal
    end
  end;
   
end;

//Darstellung der Ergebnisse
figure("background",8,"figure_position",[0 0]);
xtitle("Optimierung COACH PID-Regler","Zeit [s]","Omega_Viertel [1/4 rad/sek]");

//erst vom internen Optimierer
rKoeff=popt;
Koeffizienten_optim_Funk=popt
Guete_optim_Funktion=berechne_guetewert()
fun = ode(y0,t0,t_ziel,rechteSeite);
plot(t_ziel,fun(1,:),'r');

//dann von opt. Grad.Verf.
rKoeff=Opti_Param;
Koeffizienten_oGrad_Verf=Opti_Param
Guete_o_Grad_Verf=berechne_guetewert()
fun = ode(y0,t0,t_ziel,rechteSeite);
plot(t_ziel,fun(1,:),'b');

//zuletzt aufgezeichnete Sprungantwort plotten
plot(t_ziel,y_ziel,'m')

legend("Optimiertes Modell 1 (optim-Funktion)","Optimiertes Modell 2 (o.Grad.Verf.)","Zielfunktion",4);